varying mediump vec2 vTex;
uniform lowp sampler2D samplerFront;
uniform mediump vec2 srcStart;
uniform mediump vec2 srcEnd;
uniform mediump vec2 srcOriginStart;
uniform mediump vec2 srcOriginEnd;
uniform lowp sampler2D samplerBack;
uniform mediump vec2 destStart;
uniform mediump vec2 destEnd;
uniform mediump float seconds;
uniform mediump float angle;
uniform mediump float position;
uniform mediump float spread;
uniform mediump float cutoff;
uniform mediump float falloff;
uniform mediump float edge_fade;
uniform mediump float speed;
uniform mediump float ray1_density;
uniform mediump float ray2_density;
uniform mediump float ray2_intensity;
uniform mediump vec3 colorR;
uniform mediump float colorA;
uniform mediump float seed;
precision mediump float;

/*
Shader based on shader from Godot Shaders - the free shader library.
godotshaders.com/shader/god-rays
*/

float random(vec2 p) {
	p += min(seconds,0.0);
	return fract(sin(dot(p, vec2(12.9898,12.1414))) * 83758.5453);
}

float noise (in vec2 uv) {
    vec2 i = floor(uv);
    vec2 f = fract(uv);

    // Four corners in 2D of a tile
    float a = random(i);
    float b = random(i + vec2(1.0, 0.0));
    float c = random(i + vec2(0.0, 1.0));
    float d = random(i + vec2(1.0, 1.0));

    // Smooth Interpolation

    // Cubic Hermine Curve. Same as SmoothStep()
    vec2 u = f * f * (3.0-2.0 * f);

    // Mix 4 coorners percentages
    return mix(a, b, u.x) +
            (c - a)* u.y * (1.0 - u.x) +
            (d - b) * u.x * u.y;
}

mat2 rotate(float _angle){
    return mat2(vec2(cos(_angle), -sin(_angle)),
                vec2(sin(_angle), cos(_angle)));
}

void main(void)
{
	mediump vec2 srcOriginSize = srcOriginEnd - srcOriginStart;
	mediump vec2 uv = ((vTex - srcOriginStart) / srcOriginSize);

	// Rotate, skew and move the UVs
	vec2 transformed_uv = ( rotate(angle/360.) * (uv - position) )  / ( (uv.y + spread) - (uv.y * spread) );
	
	// Animate the ray according the the new transformed UVs
	vec2 ray1 = vec2(transformed_uv.x * ray1_density + sin(seconds * 0.5 * speed) * (ray1_density * 0.2) + seed, 1.0);
	vec2 ray2 = vec2(transformed_uv.x * ray2_density + sin(seconds * speed) * (ray1_density * 0.2) + seed, 1.0);
	
	// Cut off the ray's edges
	float cut = step(cutoff, transformed_uv.x) * step(cutoff, 1.0 - transformed_uv.x);
	ray1 *= cut;
	ray2 *= cut;
	
	// Apply the noise pattern (i.e. create the rays)
	float rays;
	
	rays = clamp(noise(ray1) + (noise(ray2) * ray2_intensity), 0., 1.);

	// Fade out edges
	rays *= smoothstep(0.0, falloff, (1.0 - uv.y)); // Bottom
	rays *= smoothstep(0.0 + cutoff, edge_fade + cutoff, transformed_uv.x); // Left
	rays *= smoothstep(0.0 + cutoff, edge_fade + cutoff, 1.0 - transformed_uv.x); // Right
	
	gl_FragColor = vec4(colorR , colorA) * rays * colorA;
}