﻿%%FRAGMENTINPUT_STRUCT%%

%%FRAGMENTOUTPUT_STRUCT%%

%%SAMPLERFRONT_BINDING%% var samplerFront : sampler;
%%TEXTUREFRONT_BINDING%% var textureFront : texture_2d<f32>;

struct ShaderParams {
	angle : f32,
	position : f32,
	spread : f32,
	cutoff: f32,
	falloff: f32,
	edge_fade: f32,
	speed: f32,
	ray1_density: f32,
	ray2_density: f32,
	ray2_intensity: f32,
	colorR: vec3<f32>,
	colorA: f32,
	seed: f32
};

%%SHADERPARAMS_BINDING%% var<uniform> shaderParams : ShaderParams;

%%C3PARAMS_STRUCT%%

%%C3_UTILITY_FUNCTIONS%%


fn random(p: vec2<f32>) -> f32 {
	var p_var = p;
	p_var = p_var + (min(c3Params.seconds, 0.));
	return fract(sin(dot(p_var, vec2<f32>(12.9898, 12.1414))) * 83758.55);
} 

fn noise(uv: vec2<f32>) -> f32 {
	let i: vec2<f32> = floor(uv);
	let f: vec2<f32> = fract(uv);
	let a: f32 = random(i);
	let b: f32 = random(i + vec2<f32>(1., 0.));
	let c: f32 = random(i + vec2<f32>(0., 1.));
	let d: f32 = random(i + vec2<f32>(1., 1.));
	let u: vec2<f32> = f * f * (3. - 2. * f);
	return mix(a, b, u.x) + (c - a) * u.y * (1. - u.x) + (d - b) * u.x * u.y;
} 

fn rotate(_angle: f32) -> mat2x2<f32> {
	return mat2x2<f32>(vec2<f32>(cos(_angle), -sin(_angle)), vec2<f32>(sin(_angle), cos(_angle)));
} 

@fragment
fn main(input : FragmentInput) -> FragmentOutput
{
	let uv: vec2<f32> = c3_srcOriginToNorm(input.fragUV);

	let transformed_uv: vec2<f32> = rotate(shaderParams.angle / 360.) * (uv - shaderParams.position) / (uv.y + shaderParams.spread - uv.y * shaderParams.spread);

	var ray1: vec2<f32> = vec2<f32>(transformed_uv.x * shaderParams.ray1_density + sin(c3Params.seconds * 0.5 * shaderParams.speed) * (shaderParams.ray1_density * 0.2) + shaderParams.seed, 1.);
	var ray2: vec2<f32> = vec2<f32>(transformed_uv.x * shaderParams.ray2_density + sin(c3Params.seconds * shaderParams.speed) * (shaderParams.ray1_density * 0.2) + shaderParams.seed, 1.);
	
	let cut: f32 = step(shaderParams.cutoff, transformed_uv.x) * step(shaderParams.cutoff, 1. - transformed_uv.x);
	ray1 = ray1 * cut;
	ray2 = ray2 * cut;
	
	var rays: f32;
	
	rays = clamp(noise(ray1) + noise(ray2) * shaderParams.ray2_intensity, 0., 1.);
	rays = rays * (smoothstep(0., shaderParams.falloff, 1. - uv.y));
	rays = rays * (smoothstep(0. + shaderParams.cutoff, shaderParams.edge_fade + shaderParams.cutoff, transformed_uv.x));
	rays = rays * (smoothstep(0. + shaderParams.cutoff, shaderParams.edge_fade + shaderParams.cutoff, 1. - transformed_uv.x));
	
	var output : FragmentOutput;
	output.color =  vec4<f32>(shaderParams.colorR, shaderParams.colorA) * rays * shaderParams.colorA;
	return output;
}